MODULE Console;

	IMPORT
      SYSTEM, Kernel, Kernel32;

CONST
  length* = 4096; (*Windows 95 limit is 1024*)
  MaxLineCount = 20;
  MaxLineLen = 256;

TYPE
  Integer = LONGINT;
  String = ARRAY OF CHAR;

VAR
  hIn, hOut   : Kernel32.HANDLE;
  char: ARRAY 2 OF CHAR;
  crlf: ARRAY 3 OF CHAR;
  text*: ARRAY length OF CHAR;
  LineNum: LONGINT;
  LineCount: LONGINT;
  DebugLine: ARRAY MaxLineCount OF ARRAY MaxLineLen OF CHAR;

PROCEDURE Length* (str: String): Integer;
VAR
  len, maxLen: Integer;
BEGIN
  maxLen := LEN(str);
  len := 0;
  WHILE (len < maxLen) & (str[len] # 0X) DO INC(len) END;
  RETURN len;
END Length;

PROCEDURE WriteString* (str: String);
VAR
  n,i,c:Integer;
BEGIN
  n:=Length(str);

  i:=0;
  WHILE (i<n) DO
    c:=ORD(str[i]);
    IF    (c>=192) & (c<=239) THEN c:=c-64;
    ELSIF (c>=240) & (c<=255) THEN c:=c-16;
    ELSIF (c=168)             THEN c:=240;
    ELSIF (c=184)             THEN c:=241;
    END;
    str[i]:=CHR(c);
    INC(i);
  END;

  Kernel32.WriteFile(hOut, str, n, n, NIL);
END WriteString;

PROCEDURE Write* (ch: CHAR);
BEGIN
  char[0] := ch; WriteString(char);
END Write;

PROCEDURE WriteLn* ;
BEGIN
  WriteString(crlf);
END WriteLn;

PROCEDURE ReadString* (VAR str: String);
VAR
  n,i,c:Integer;
BEGIN
  n:=LEN(str);
  Kernel32.ReadFile(hIn, str, n, n, NIL);
  str[n-2]:=0X; (* - 0AX 0DX *)

  i:=0;
  WHILE i<n DO
    c:=ORD(str[i]);
    IF    (c>=192) & (c<=239) THEN c:=c+16;
    ELSIF (c=240)             THEN c:=168;
    ELSIF (c=241)             THEN c:=184;
    ELSIF (c=168)             THEN c:=232;
    END;
    str[i]:=CHR(c);
    INC(i);
  END;

END ReadString;

PROCEDURE Append*(VAR to: ARRAY OF CHAR; this: ARRAY OF CHAR);
VAR
  i, j, l: LONGINT;
BEGIN
  i := 0;
  WHILE to[i] # 0X DO INC(i) END;
  l := LEN(to)-1; j := 0;
  WHILE (i < l) & (this[j] # 0X) DO
	to[i] := this[j]; INC(i); INC(j)
  END;
  to[i] := 0X
END Append;

PROCEDURE Get();
VAR
adr: Kernel32.ADDRESS;
  i: LONGINT;
  ch: CHAR;
BEGIN
  i := 0;
  adr := Kernel32.GetCommandLine();
  SYSTEM.GET(adr, ch);
  WHILE (ch # 0X) & (i < length - 1) DO
	text[i] := ch;
	INC(i);
	INC(adr);
	SYSTEM.GET(adr, ch);
  END;
  text[i] := 0X;
END Get;

(*######################################################################*)

PROCEDURE OutputString(s: ARRAY OF CHAR);
VAR
  i,j,k: LONGINT;
  a: ARRAY 10 OF CHAR;
BEGIN
  Append(DebugLine[LineNum],s);
END OutputString;

PROCEDURE Tab*();
VAR
  tab: ARRAY 2 OF CHAR;
BEGIN
  tab[0]:=9X;
  tab[1]:=0X;
  OutputString(tab);
END Tab;

PROCEDURE Str*(s: ARRAY OF CHAR);
BEGIN
OutputString(s);
END Str;

PROCEDURE Ln*;
VAR
  i,j: LONGINT;
  a: ARRAY 22 OF CHAR;
BEGIN
IF LineNum<MaxLineCount-1 THEN
    INC(LineNum);
  ELSE
    FOR i:=0 TO MaxLineCount-2 DO
      COPY(DebugLine[i+1],DebugLine[i]);
    END;
    DebugLine[MaxLineCount-1,0]:=0X;
    DebugLine[MaxLineCount-1,1]:=0X;
  END;
  INC(LineCount);
END Ln;

PROCEDURE IntToStr(i: LONGINT; VAR s: ARRAY OF CHAR);
	VAR k, j, x0: LONGINT; a: ARRAY 10 OF CHAR;
BEGIN
    k := 0;
	IF i < 0 THEN
		IF i = MIN(LONGINT) THEN
			COPY("-2147483648", s); RETURN
		ELSE
			x0 := -i
		END
	ELSE
		x0 := i
	END;
	REPEAT
		a[k] := CHR(x0 MOD 10 + 30H); x0 := x0 DIV 10; INC(k)
	UNTIL x0 = 0;
	IF i < 0 THEN
		s[0] := "-"; j := 1
	ELSE
		j := 0
	END;
	REPEAT DEC(k); s[j] := a[k]; INC(j) UNTIL k = 0;
	s[j] := 0X
END IntToStr;

PROCEDURE Int*(x: LONGINT);
VAR
  s: ARRAY 16 OF CHAR;
BEGIN
IntToStr(x, s);
  OutputString(s);
END Int;

PROCEDURE Hex*(x: LONGINT);
VAR
  s: ARRAY 16 OF CHAR;
  d, i: LONGINT;
BEGIN
	s[8] := 0X;
	FOR i := 7 TO 0 BY -1 DO
		d := x MOD 16;
		IF d < 10 THEN s[i] := CHR(d + ORD("0")) ELSE s[i] := CHR(d-10+ORD("A")) END;
		x := x DIV 16
	END;
	OutputString(s);
END Hex;

PROCEDURE Assert* (F: BOOLEAN);
VAR
  S: ARRAY 10 OF CHAR;
  i,j: LONGINT;
BEGIN
  IF F THEN RETURN END;
  WriteString("Exception:");WriteLn();WriteLn();
  IF LineNum<MaxLineCount THEN j:=LineNum ELSE j:=MaxLineCount-2 END;
  FOR i:=0 TO j DO
    WriteString(DebugLine[i]); WriteLn();
  END;
  WriteLn();
  WriteString("Press ENTER to continue...");
  ReadString(S);
  Kernel.Shutdown(0);
END Assert;

PROCEDURE LogMessage* (N: LONGINT);
VAR
  i,j,k: LONGINT;
BEGIN
  WriteString("Message:");WriteLn();WriteLn();
  IF LineNum<MaxLineCount THEN j:=LineNum ELSE j:=MaxLineCount-2 END;
  IF N<MaxLineCount THEN k:=MaxLineCount-N ELSE k:=0 END;
  FOR i:=k TO j DO
    WriteString(DebugLine[i]); WriteLn();
  END;
  WriteLn();
END LogMessage;

BEGIN
  crlf[0] := 0DX; crlf[1] := 0AX; crlf[2] := 0X; char[1] := 0X;
  LineNum:=0;
  LineCount:=0;
  hIn  := Kernel32.GetStdHandle(Kernel32.STDInput);
  hOut := Kernel32.GetStdHandle(Kernel32.STDOutput);
  Get();
END Console.

