(*#####################################################*)
(*#   Forms for Oberon Portable Compiler and Linker   #*)
(*#     ( https://sourceforge.net/projects/opcl )     #*)
(*# Version 1.0 for Win32: 9x,NT,Me,2k,XP,2k3,Vista,7 #*)
(*#  It is free for commercial & non-commercial use.  #*)
(*#                                                   #*)
(*# Copyright (C) 2011-2012, Serg Atantsov            #*)
(*#     e-mail Web_2.0@mail.ru     ICQ 200972129      #*)
(*#                                                   #*)
(*# Copyright (C) 2007-2012, Oleg N. Cher             #*)
(*#  www.zx.oberon2.ru e-mail allot@bk.ru ICQ 2208403 #*)
(*#####################################################*)

MODULE Forms;

IMPORT
  SYSTEM, W:=Windows, M:=Messages, C:=Controls, G:=Graphics;

VAR
  Application*: TApplication;

CONST
  poLeftTop*       = 1;
  poLeftCenter*    = 2;
  poLeftBottom*    = 3;
  poCenterTop*     = 4;
  poCenterCenter*  = 5;
  poCenterBottom*  = 6;
  poRightTop*      = 7;
  poRightCenter*   = 8;
  poRightBottom*   = 9;
    
TYPE
  TFormList*       = POINTER TO RECORD
    Next     : TFormList;
    Pointer* : SYSTEM.PTR;
    Handle*  : W.HWND;
    
    PROCEDURE &Create;
    BEGIN
      Next:=NIL;
    END Create;
    
    PROCEDURE Find*(H : W.HWND): SYSTEM.PTR;
    VAR
      Result: SYSTEM.PTR;
      List  : TFormList;
    BEGIN
      Result:=NIL;
      List:=Next;
      WHILE (List#NIL) & (Result=NIL) DO
        IF List.Handle=H THEN
          Result:=List.Pointer;
        END;
        List:=List.Next;
      END;
      RETURN Result;
    END Find;
    
    PROCEDURE Add*(P : SYSTEM.PTR; H : W.HWND);
    VAR
      New   : TFormList;
    BEGIN
      IF Find(H)#NIL THEN RETURN END;
      NEW(New);
      New.Pointer:=P;
      New.Next:=Next;
      New.Handle:=H;
      Next:=New;
    END Add;

  END;
  
  TForm*       = POINTER TO RECORD (C.TWinControl);
    
    Canvas* : G.TCanvas; 
    DoubleBuffered* : BOOLEAN;
    Font*   : G.TFont;
    Color*  : LONGINT;
    Position*: LONGINT;
    
    Loaded  : BOOLEAN;
    Showing : BOOLEAN;
    
    PROCEDURE Create*(Owner: C.TWinControl);
    BEGIN
      Create^(Owner);       
      Style:=W.WS_OVERLAPPEDWINDOW+W.WS_CLIPCHILDREN;
      IF Owner # NIL THEN 
        Style:=W.WS_OVERLAPPEDWINDOW+W.WS_CLIPSIBLINGS; 
      END;      
      Handle := W.CreateWindowEx(
         W.WS_EX_APPWINDOW,
         Application.ClassName, Caption, Style,
         Left, Top, Width, Height, 0, 0, W.hInstance, 0
      );      
    END Create;
      
    PROCEDURE CreateForm*(Owner: C.TWinControl);
    BEGIN      
      Create(Owner);
      
      NEW(Canvas,Handle);
      Color:=G.clBtnFace;
      
      NEW(Font);
      Font.Create("Tahoma");
      
      W.SetWindowPos(Handle, W.HWND_NOTOPMOST,Left,Top,Width,Height,W.SWP_HIDEWINDOW);
      
      IF Owner # NIL THEN 
        W.SetParent(Handle,Owner.Handle);
      END;
    END CreateForm;
    
    PROCEDURE WindowProc*(msg,wParam,lParam: LONGINT);
    VAR
      Control: SYSTEM.PTR;
      List   : C.TControlList;
      i      : LONGINT;
      R      : W.RECT;
      O      : C.TWinControl;     
    BEGIN
      WindowProc^(msg,wParam,lParam);  
      CASE msg OF
      
         M.WM_PAINT:  
                            Canvas.Create(DoubleBuffered);
                            Canvas.SetColor(Color);
                            Paint;
                            Canvas.Release(); 
      |  M.WM_COMMAND:      
                            Control:=C.ControlList.Find(SELF,wParam);
                            IF (Control # NIL) THEN
                              Control(C.TWinControl).WindowProc(msg,wParam,lParam);
                            END;
      |  M.WM_ACTIVATE:     
                            IF ~Loaded THEN 
                              i:=1;
                              WHILE i<=C.ID DO
                                Control:=C.ControlList.Find(SELF,i);
                                IF (Control # NIL) THEN                               
                                  Control(C.TWinControl).Update;
                                END;
                                INC(i);
                              END;
                              Loaded:=TRUE;
                            END;
      |  M.WM_SIZE, M.WM_MOVE   :  
                            W.GetWindowRect(Handle,R); 
                            Width  := R.right-R.left;
                            Height := R.bottom-R.top;
                            Left:= R.left;
                            Top := R.top;                            
                            IF Owner # NIL THEN
                              W.GetWindowRect(Owner.Handle,R);                              
                              Left:=Left-R.left-W.GetSystemMetrics(W.SM_CXFRAME);
                              Top :=Top -R.top -W.GetSystemMetrics(W.SM_CYCAPTION)*2;
                            END;
                            IF msg=M.WM_SIZE THEN                            
                              i:=1;
                              WHILE i<=C.ID DO
                                Control:=C.ControlList.Find(SELF,i);
                                IF (Control # NIL) THEN                               
                                  Control(C.TWinControl).Update;
                                END;
                                INC(i);
                              END;
                            END;

      ELSE
      END;      
    END WindowProc;

    PROCEDURE Paint*;
    BEGIN
    END Paint;
    
    PROCEDURE FormShow*;
    BEGIN
    END FormShow;
    
    PROCEDURE SetPosition*(Pos: LONGINT);
    VAR
      r: W.RECT;
      tp: W.APPBARDATA;
      dl,dt,dr,db: LONGINT;
    BEGIN
      W.GetWindowRect(W.GetDesktopWindow(),r);
      tp.cbSize:=SIZE(W.APPBARDATA);
      dl:=0; dt:=0; dr:=0; db:=0;
      IF W.SHAppBarMessage(W.ABM_GETTASKBARPOS, tp) # 0 THEN
        CASE tp.uEdge OF
           W.ABE_LEFT  : dl:=tp.rc.right-tp.rc.left;
        |  W.ABE_TOP   : dt:=tp.rc.bottom-tp.rc.top;
        |  W.ABE_RIGHT : dr:=tp.rc.right-tp.rc.left;
        |  W.ABE_BOTTOM: db:=tp.rc.bottom-tp.rc.top;
        ELSE
        END;
      ELSE
      END;
      CASE Pos OF
         poLeftTop        : Left:=0+dl; Top:=0+dt;
      |  poLeftCenter     : Left:=0+dl; Top:=((r.bottom-Height) DIV 2)+ (db+dt) DIV 2;
      |  poLeftBottom     : Left:=0+dl; Top:=r.bottom-Height-db;
      |  poCenterTop      : Left:=((r.right-Width) DIV 2)+(dr+dl) DIV 2; Top:=0+dt;
      |  poCenterCenter   : Left:=((r.right-Width) DIV 2)+(dr+dl) DIV 2; Top:=((r.bottom-Height) DIV 2)+(db+dt) DIV 2;
      |  poCenterBottom   : Left:=((r.right-Width) DIV 2)+(dr+dl) DIV 2; Top:=r.bottom-Height-db;
      |  poRightTop       : Left:=r.right-Width-dr; Top:=0+dt;
      |  poRightCenter    : Left:=r.right-Width-dr; Top:=((r.bottom-Height) DIV 2)+ (db+dt) DIV 2;
      |  poRightBottom    : Left:=r.right-Width-dr; Top:=r.bottom-Height-db;
      ELSE
      END;
    END SetPosition;
    
    PROCEDURE Show*;
    BEGIN
      IF ~Showing & (Position # 0) THEN
        SetPosition(Position);
        Showing:=TRUE;
      END;
      W.SetWindowPos(Handle, W.HWND_NOTOPMOST,Left,Top,Width,Height,W.SWP_SHOWWINDOW);
      IF Owner # NIL THEN 
        W.ShowWindow(Handle, W.SW_SHOW); 
      END;
      W.UpdateWindow(Handle);
      FormShow;
    END Show;

    PROCEDURE Hide*;
    BEGIN
      W.SetWindowPos(Handle, W.HWND_NOTOPMOST,Left,Top,Width,Height,W.SWP_HIDEWINDOW);
      IF Owner # NIL THEN 
        W.ShowWindow(Handle, W.SW_HIDE); 
      END;
      W.UpdateWindow(Handle);
    END Hide;

  END;
  
  TApplication = POINTER TO RECORD

    ClassName*  : ARRAY 32 OF CHAR;
    MainForm*   : SYSTEM.PTR;
    List*       : TFormList;
    ActiveForm* : SYSTEM.PTR;
    fTerminated : BOOLEAN;

    PROCEDURE Initialize*;
    VAR
      class     : W.WNDCLASSEX;
    BEGIN
      NEW(List);
      ClassName           :='FORM';
      class.cbSize        := SIZE(W.WNDCLASSEX);
      class.style         := W.CS_VREDRAW + W.CS_HREDRAW;
      class.lpfnWndProc   := WndProc;
      class.cbClsExtra    := 0;
      class.cbWndExtra    := 0;
      class.hInstance     := W.hInstance;
      class.hIcon         := W.LoadIcon(W.hInstance,W.IDI_APPLICATION);
      class.hCursor       := W.LoadCursor(0,SYSTEM.VAL(W.LPSTR,W.IDC_ARROW));
      class.hbrBackground := W.NULL;
      class.lpszMenuName  := W.NULL;
      class.lpszClassName := SYSTEM.ADR(Application.ClassName);
      class.hIconSm       := W.NULL;
      W.RegisterClassEx(class);
    END Initialize;

    PROCEDURE Exec*(Form: SYSTEM.PTR; msg: LONGINT; wParam: W.WPARAM; lParam: W.LPARAM);
    BEGIN
      IF MainForm=NIL THEN RETURN END;
      Form(TForm).WindowProc(msg,wParam,lParam);
    END Exec;

    PROCEDURE CreateForm*(Form: SYSTEM.PTR; Owner: C.TWinControl);
    BEGIN
      IF MainForm=NIL THEN MainForm:=Form; END;
      Form(TForm).CreateForm(Owner);
      List.Add(Form,Form(TForm).Handle);
    END CreateForm;

    PROCEDURE Run*;
    BEGIN
      IF MainForm=NIL THEN RETURN END;
      MainForm(TForm).Show;
      REPEAT
        ProcessMessages;
      UNTIL fTerminated;
      W.UnregisterClass('FORM',W.hInstance);
      W.Shutdown();
    END Run;

    PROCEDURE ProcessMessages*;
    VAR
      msg : M.MSG;
    BEGIN
      W.GetMessage(msg,0,0,0);
      W.TranslateMessage(msg);
      W.DispatchMessage(msg);
    END ProcessMessages;

    PROCEDURE Terminate*;
    BEGIN
      fTerminated:=TRUE;
    END Terminate;

  END;
    
PROCEDURE [WINAPI] WndProc* (wnd: W.HWND; msg: LONGINT; wParam: W.WPARAM; lParam: W.LPARAM): W.LRESULT;
BEGIN

  Application.ActiveForm:= Application.List.Find(wnd);
  IF Application.ActiveForm # NIL THEN Application.Exec(Application.ActiveForm,msg,wParam,lParam); END;
  
  CASE msg OF
     M.WM_CLOSE: 
                  IF wnd=Application.MainForm(TForm).Handle THEN
                    Application.fTerminated:=TRUE;
                    W.PostQuitMessage(0);
                  ELSE
                    W.ShowWindow(wnd, W.SW_HIDE);
                  END;
                  RETURN 0;
  ELSE
    RETURN W.DefWindowProc(wnd, msg, wParam, lParam)
  END;
  RETURN 0

END WndProc;
    
BEGIN
  NEW(Application);
END Forms.